@@ -59,7 +59,7 @@
 # define S_ISGID 02000
 #endif
 
-/* S_ISVTX is a common extension to POSIX.1.  */
+/* S_ISVTX is a common extension to POSIX.  */
 #ifndef S_ISVTX
 # define S_ISVTX 01000
 #endif
@@ -113,7 +113,7 @@
 # define S_IRWXO (S_IROTH | S_IWOTH | S_IXOTH)
 #endif
 
-/* S_IXUGO is a common extension to POSIX.1.  */
+/* S_IXUGO is a common extension to POSIX.  */
 #if !S_IXUGO
 # define S_IXUGO (S_IXUSR | S_IXGRP | S_IXOTH)
 #endif
@@ -243,10 +243,9 @@ char *alloca ();
    - Its arg may be any int or unsigned int; it need not be an unsigned char.
    - It's guaranteed to evaluate its argument exactly once.
    - It's typically faster.
-   Posix 1003.2-1992 section 2.5.2.1 page 50 lines 1556-1558 says that
-   only '0' through '9' are digits.  Prefer ISDIGIT to ISDIGIT_LOCALE unless
-   it's important to use the locale's definition of `digit' even when the
-   host does not conform to Posix.  */
+   POSIX says that only '0' through '9' are digits.  Prefer ISDIGIT to
+   ISDIGIT_LOCALE unless it's important to use the locale's definition
+   of `digit' even when the host does not conform to POSIX.  */
 #define ISDIGIT(c) ((unsigned) (c) - '0' <= 9)
 
 #ifndef PARAMS
@@ -554,3 +553,13 @@ enum
     }						\
   while (0)
 #endif
+
+#ifndef EOVERFLOW
+# define EOVERFLOW EINVAL
+#endif
+
+#if ! HAVE_FSEEKO && ! defined fseeko
+# define fseeko(s, o, w) ((o) == (long) (o)		\
+			  ? fseek (s, o, w)		\
+			  : (errno = EOVERFLOW, -1))
+#endif
